@@ -52,6 +52,8 @@ SWITCH_BEGIN_EXTERN_C
 typedef struct {
 	switch_rtp_hdr_t header;
 	char body[SWITCH_RTP_MAX_BUF_LEN+4+sizeof(char *)];
+	switch_rtp_hdr_ext_t *ext;
+	char *ebody;
 } switch_rtp_packet_t;
 
 typedef enum {
